// PtDevice.cpp: PTfoCXNX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "PtDevice.h"
#include <TChar.h>
#include <MmSystem.h>


#pragma comment(lib, "WinMm.lib")


/////////////////////////////////////////////////////////////////////////////
// PT}l[WNX
/////////////////////////////////////////////////////////////////////////////

CPtManager::CPtManager(void)
	: m_pPtBus(NULL)
{
	// ^C}
	::timeBeginPeriod(1UL);

	::BON_ASSERT(EnumerateBoards());
}

CPtManager::~CPtManager(void)
{
	::BON_ASSERT(ReleaseInstance());

	// ^C}J
	::timeEndPeriod(1UL);
}

const DWORD CPtManager::GetTotalTunerNum(const Device::ISDB dwTunerType)
{
	// ĂPT1̃`[iԂ
	return m_Boards.size() * CPtBoard::TUNER_NUM;
}

const DWORD CPtManager::GetActiveTunerNum(const Device::ISDB dwTunerType)
{
	// gpĂPT1̃`[iԂ
	DWORD dwActiveNum = 0UL;

	for(DWORD dwIndex = 0UL ; dwIndex < m_Boards.size() ; dwIndex++){
		dwActiveNum += m_Boards[dwIndex]->GetActiveTunerNum(dwTunerType);
		}

	return dwActiveNum;
}

CPtTuner * CPtManager::OpenTuner(const Device::ISDB dwTunerType)
{
	// gp̃`[iĕԂ
	for(DWORD dwBoardID = 0UL ; dwBoardID < m_Boards.size() ; dwBoardID++){
		CPtTuner *pTuner = m_Boards[dwBoardID]->OpenTuner(dwTunerType);
		if(pTuner)return pTuner;
		}

	return NULL;
}

const bool CPtManager::EnumerateBoards(void)
{
	static Library PtApi;

	// PT1{[h񋓂
	try{
		// BusCX^Xt@Ng[擾
		Bus::NewBusFunction pfnBusFactory = PtApi.Function();
		if(!pfnBusFactory)throw ::BON_EXPECTION(TEXT("SDK_EARTHSOFT_PT1.dll[hł܂"));

		// BusCX^X
		if(pfnBusFactory(&m_pPtBus) != STATUS_OK)throw ::BON_EXPECTION(TEXT("BusCX^Xs"));
		
		// PT1{[h
		Bus::DeviceInfo DeviceInfo[8];
		DWORD dwDeviceInfoNum = sizeof(DeviceInfo) / sizeof(*DeviceInfo);
		
		if(m_pPtBus->Scan(DeviceInfo, &dwDeviceInfoNum) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT{[h񋓎s"));

		// PT1foCXCX^X
		for(DWORD dwBoardID = 0UL ; dwBoardID < dwDeviceInfoNum ; dwBoardID++){
			// SĂPT1{[h̃foCXCX^X𐶐
			Device *pNewDevice = NULL;
			Device2 *pNewDevice2 = NULL;
			
			// Device2̃CX^X𐶐ȂDevice::Open()ɎsII

			if(m_pPtBus->NewDevice(&DeviceInfo[dwBoardID], &pNewDevice, &pNewDevice2) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXCX^Xs"));
			
			// Xgɒǉ
			m_Boards.push_back(new CPtBoard(dwBoardID, pNewDevice, &DeviceInfo[dwBoardID]));
			}
		}
	catch(CBonException &Exception){
		// G[
		Exception.Notify();
		ReleaseInstance();
		return false;
		}
	
	return true;
}

const bool CPtManager::ReleaseInstance(void)
{
	// PT{[h̃CX^XJ
	for(DWORD dwBoardID = 0UL ; dwBoardID < m_Boards.size() ; dwBoardID++){
		delete m_Boards[dwBoardID];
		}
		
	m_Boards.clear();

	// oXCX^XJ
	if(m_pPtBus){
		m_pPtBus->Delete();
		m_pPtBus = NULL;
		}

	return true;
}


/////////////////////////////////////////////////////////////////////////////
// PT{[hNX
/////////////////////////////////////////////////////////////////////////////

CPtBoard::CPtBoard(const DWORD dwBoardID, Device *pDevice, const Bus::DeviceInfo *pDeviceInfo)
	: m_dwBoardID(dwBoardID)
	, m_pDevice(pDevice)
	, m_DeviceInfo(*pDeviceInfo)
{
	::BON_ASSERT(m_pDevice != NULL);

	// `[iCX^X
	for(DWORD dwTunerID = 0UL ; dwTunerID < TUNER_NUM ; dwTunerID++ ){
		m_TunerList.push_back(new CPtTuner(*this, dwTunerID, Device::ISDB_S));
		m_TunerList.push_back(new CPtTuner(*this, dwTunerID, Device::ISDB_T));
		}
}

CPtBoard::~CPtBoard(void)
{
	// `[iCX^XJ
	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		delete m_TunerList[dwIndex];
		}

	// foCXCX^XJ
	m_pDevice->Delete();
}

CPtTuner * CPtBoard::OpenTuner(const Device::ISDB dwTunerType)
{
	CPtTuner *pOpenTuner = NULL;

	// I[v`[i
	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		if(!m_TunerList[dwIndex]->m_bIsOpen && (m_TunerList[dwIndex]->m_dwTunerType == dwTunerType)){
			// gp`[iނv
			pOpenTuner = m_TunerList[dwIndex];
			}
		}

	if(!pOpenTuner)return NULL;
	
	if(!GetOpenTunerNum()){
		// S`[igp̏ꍇ̓{[hI[v
		if(!OpenBoard())return NULL;
		}
		
	// I[vtOZbg
	pOpenTuner->m_bIsOpen = true;

	return pOpenTuner;
}

void CPtBoard::CloseTuner(CPtTuner *pTuner)
{
	if(!IsValidTuner(pTuner))return;
	if(!pTuner->m_bIsOpen)return;

	// Xg[~AI[vtONA
	pTuner->StopStream();
	pTuner->m_bIsOpen = false;

	if(!GetOpenTunerNum()){
		// S`[igp̏ꍇ̓{[hN[Y
		CloseBoard();
		}
}

const DWORD CPtBoard::GetActiveTunerNum(const Device::ISDB dwTunerType)
{
	// {[hO[obNMutex݊mF	u/oXԍ/Xbgԍ/t@NV/PT1/vZXID/v
	TCHAR szMutexName[1024];
	::_stprintf(szMutexName, TEXT("/%lu/%lu/%lu/PT1/"), m_DeviceInfo.Bus, m_DeviceInfo.Slot, m_DeviceInfo.Function);

	if(!CSmartMutex::IsExist(szMutexName)){
		// {[hgp̏ꍇ
		return 0UL;
		}
	else if(!GetOpenTunerNum()){
		// vZX{[hgpĂꍇ
		return TUNER_NUM;
		}
	else{
		// vZX{[hgpĂꍇ
		DWORD dwActiveNum = 0UL;

		for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
			if(m_TunerList[dwIndex]->m_dwTunerType == dwTunerType){
				if(m_TunerList[dwIndex]->m_bIsOpen)dwActiveNum++;
				}
			}
		
		// gp̃`[iԂ
		return dwActiveNum;
		}
}

const bool CPtBoard::StartStream(CPtTuner *pTuner)
{
	if(!IsValidTuner(pTuner))return false;
	if(pTuner->m_bIsStream)return true;

	if(!GetStreamTunerNum()){
		// fBXpb`XbhJn
		::BON_ASSERT(m_DispatchThread.StartThread(this, &CPtBoard::DispatchThread, NULL, false ), TEXT("fBXpb`XbhNs"));

		// Xg[~OXbhJn
		::BON_ASSERT(m_StreamingThread.StartThread(this, &CPtBoard::StreamingThread, NULL, true), TEXT("Xg[~OXbhNs"));
		}

	// Xg[MtOZbg
	pTuner->m_bIsStream = true;

	return true;
}

void CPtBoard::StopStream(CPtTuner *pTuner)
{
	if(!IsValidTuner(pTuner))return;
	if(!pTuner->m_bIsStream)return;

	// Xg[MtONA
	pTuner->m_bIsStream = false;

	if(!GetStreamTunerNum()){
		// Xg[~OXbh~
		m_StreamingThread.EndThread();
		
		// fBXpb`Xbh~
		m_DispatchThread.EndThread();
		}
}

const bool CPtBoard::SetLnbPower(CPtTuner *pTuner, const bool bEnable)
{
	if(!IsValidTuner(pTuner))return false;
	if(pTuner->m_dwTunerType != Device::ISDB_S)return false;

	// XVOLNBdԎ擾
	const bool bLastState = (GetLnbOnTunerNum())? true : false;
	
	// LNBdԍXV
	pTuner->m_bIsLnbOn = bEnable;

	// XVLNBdԎ擾
	const bool bNewState = (GetLnbOnTunerNum())? true : false;
	if(bLastState == bNewState)return true;			

	// LNBdԐݒ
	::BON_ASSERT(m_pDevice->SetLnbPower((bNewState)? Device::LNB_POWER_15V : Device::LNB_POWER_OFF) == STATUS_OK, TEXT("PT`[iLNBd䎸s"));
	
	return true;
}

const bool CPtBoard::IsValidTuner(const CPtTuner *pTuner) const
{
	// Lȃ`[iCX^X̗LԂ
	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		if(m_TunerList[dwIndex] == pTuner)return true;
		}

	return false;
}

const DWORD CPtBoard::GetOpenTunerNum(void) const
{
	// I[vς݃`[iԂ
	DWORD dwOpenNum = 0UL;

	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		if(m_TunerList[dwIndex]->m_bIsOpen)dwOpenNum++;
		}

	return dwOpenNum;
}

const DWORD CPtBoard::GetLnbOnTunerNum(void) const
{
	// LNBdOÑ`[iԂ
	DWORD dwLnbOnNum = 0UL;

	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		if((m_TunerList[dwIndex]->m_dwTunerType == Device::ISDB_S) && m_TunerList[dwIndex]->m_bIsLnbOn)dwLnbOnNum++;
		}

	return dwLnbOnNum;
}

const DWORD CPtBoard::GetStreamTunerNum(void) const
{
	// Xg[M̃`[iԂ
	DWORD dwOpenNum = 0UL;

	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		if(m_TunerList[dwIndex]->m_bIsStream)dwOpenNum++;
		}

	return dwOpenNum;
}

const bool CPtBoard::OpenBoard(void)
{
	// {[hI[v
	try{
		// foCXI[v
		if(m_pDevice->Open() != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXI[vs"));
		
		// `[i[uI/Cl[uv
		if(m_pDevice->SetTunerPowerReset(Device::TUNER_POWER_ON_RESET_ENABLE) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT`[iZbgڍss"));

		// 20msEFCg
		::Sleep(20UL);

		// `[i[uI/fBZ[uv
		if(m_pDevice->SetTunerPowerReset(Device::TUNER_POWER_ON_RESET_DISABLE) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT`[iZbgs"));

		// 1msEFCg
		::Sleep(1UL);
		
		for(DWORD dwTunerID = 0UL ; dwTunerID < TUNER_NUM ; dwTunerID++){
			// `[iW[
			if(m_pDevice->InitTuner(dwTunerID) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT`[is"));

			// `[iX[v
			for(DWORD dwTunerType = 0UL ; dwTunerType < Device::ISDB_COUNT ; dwTunerType++){
				if(m_pDevice->SetTunerSleep(dwTunerID, (Device::ISDB)dwTunerType, false) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT`[iȓd͉s"));
				}
			}
		
		// {[hO[obNMutex쐬	u/oXԍ/Xbgԍ/t@NV/PT1/v
		TCHAR szMutexName[1024];
		::_stprintf(szMutexName, TEXT("/%lu/%lu/%lu/PT1/"), m_DeviceInfo.Bus, m_DeviceInfo.Slot, m_DeviceInfo.Function);
		if(!m_BoardLock.Create(szMutexName))throw ::BON_EXPECTION(TEXT("PT{[hO[obNMutex쐬s"));
		}
	catch(CBonException &Exception){
		// G[
		Exception.Notify();
		CloseBoard();		
		return false;
		}

	return true;
}

void CPtBoard::CloseBoard(void)
{
	// {[hN[Y
	try{
		// `[iX[vڍs
		for(DWORD dwTunerID = 0UL ; dwTunerID < TUNER_NUM ; dwTunerID++){
			for(DWORD dwTunerType = 0UL ; dwTunerType < Device::ISDB_COUNT ; dwTunerType++){
				if(m_pDevice->SetTunerSleep(dwTunerID, (Device::ISDB)dwTunerType, false) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT`[iȓd͈ڍss"));
				}
			}	

		// {[hN[Y
		if(m_pDevice->Close() != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXN[Ys"));
		}
	catch(CBonException &Exception){
		// G[
		Exception.Notify();
		}

	// {[hbNMutexJ
	m_BoardLock.Close();
}

void CPtBoard::StreamingThread(CSmartThread<CPtBoard> *pThread, bool &bKillSignal, PVOID pParam)
{
	::BON_TRACE(TEXT("CPtBoard::StreamingThread() Start\n"));

	struct PT_DMABUF_SET
	{
		DWORD *pdwPacket;			// obt@|C^
		DWORD *pdwEndMark;			// ]}[Ñ|C^
		PT_DMABUF_SET *pNextSet;	// ̃obt@Zbgւ̃|C^
	};

	// DMAobt@Zbg
	PT_DMABUF_SET aDmaBufSet[DMABUF_SIZE * DMABUF_NUM];
	PT_DMABUF_SET *pDmaBufSet = aDmaBufSet;

	try{
		// fBXpb`JE^NA
		for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
			m_TunerList[dwIndex]->m_dwStreamCounter = 0UL;
			}

		// fBXpb`L[NA
		while(!m_DispatchQueue.empty())m_DispatchQueue.pop();
		
		// DMAobt@TCYݒ
		const static Device::BufferInfo BufInfo = {DMABUF_SIZE, DMABUF_NUM, DMABUF_SIZE};
		if(m_pDevice->SetBufferInfo(&BufInfo) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMAobt@mێs"));

		// DMAobt@|C^擾(󃊃Xg`)
		for(DWORD dwBuffIndex = 0UL ; dwBuffIndex < DMABUF_NUM ; dwBuffIndex++){
			// obt@|C^擾		
			DWORD *pdwPointer = NULL;
			if(m_pDevice->GetBufferPtr(dwBuffIndex, (PVOID*)&pdwPointer) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMAobt@|C^擾s"));
		
			for(DWORD dwBlockIndex = 0UL ; dwBlockIndex < DMABUF_SIZE ; dwBlockIndex++){
				// obt@|C^Zbg
				pDmaBufSet->pdwPacket = &pdwPointer[0]; 
			
				// ]}[N|C^Zbg
				pDmaBufSet->pdwEndMark = &pdwPointer[4096 * 511 / 4 - 1];
				pdwPointer = &pdwPointer[4096 * 511 / 4];
		
				// ̃Zbgւ̃|C^Zbg
				pDmaBufSet->pNextSet = &pDmaBufSet[1];
				pDmaBufSet++;
				}
			}

		// ŏIvf̎Zbg|C^␳
		aDmaBufSet[sizeof(aDmaBufSet) / sizeof(*aDmaBufSet) - 1].pNextSet = aDmaBufSet;
		pDmaBufSet = aDmaBufSet;
	
		// ]JE^Zbg
		if(m_pDevice->ResetTransferCounter() != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMA]JE^Zbgs"));

		// ]JE^CNg
		if(m_pDevice->IncrementTransferCounter() != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMA]JE^CNgs"));

		// DMA]Jn
		if(m_pDevice->SetTransferEnable(true) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMA]Jns"));

		// DMA][v
		while(!bKillSignal){
		
			// ]}[J[Zbg
			*pDmaBufSet->pdwEndMark = 0UL;
		
			// ]JE^CNg
			if(m_pDevice->IncrementTransferCounter() != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMA]JE^CNgs"));

			// ]EFCg
			while(!bKillSignal){

				// ubN]`FbN
				if(*pDmaBufSet->pdwEndMark){
					CBlockLock AutoLock(&m_DispatcLock);

					// fBXpb`L[ɓ
					if(m_DispatchQueue.size() < (DMABUF_SIZE * DMABUF_NUM)){
						m_DispatchQueue.push(pDmaBufSet->pdwPacket);
						m_DispatchThread.ResumeThread();
						}
					else{
						::BON_TRACE(TEXT("fBXpb`L[I[o[t[\n"));
						}
					
					// DMAobt@ZbgXV
					pDmaBufSet = pDmaBufSet->pNextSet;
					
					break;
					}
				else{
					// EFCg
					::Sleep(10);
					}
				}
			}

		// DMA]~
		if(m_pDevice->SetTransferEnable(false) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMA]~s"));

		// DMAobt@J
		if(m_pDevice->SetBufferInfo(NULL) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMAobt@Js"));
		}
	catch(CBonException &Exception){
		// G[
		Exception.Notify();
		return;
		}

	::BON_TRACE(TEXT("CPtBoard::StreamingThread() End\n"));
}

void CPtBoard::DispatchThread(CSmartThread<CPtBoard> *pThread, bool &bKillSignal, PVOID pParam)
{
	::BON_TRACE(TEXT("CPtBoard::DispatchThread() Start\n"));

	CMediaData DispatchBuffer(4096UL * 511UL);
	const DWORD *pdwData;

	while(!bKillSignal){
		// t@C
		while(1){
			// fBXpb`f[^`FbN
			m_DispatcLock.Lock();
			pdwData = (!m_DispatchQueue.empty())? m_DispatchQueue.front() : NULL;
			m_DispatcLock.Unlock();
			if(!pdwData)break;

			// fBXpb`f[^Rs[(DMAobt@CPULbVȂ)
			DispatchBuffer.SetData((const BYTE *)pdwData, 4096UL * 511UL);
			
			// }CNpPbgfBXpb`
			DispatchStream((const DWORD *)DispatchBuffer.GetData(), DispatchBuffer.GetSize() / 4UL);

			// fBXpb`f[^o
			m_DispatcLock.Lock();
			m_DispatchQueue.pop();
			m_DispatcLock.Unlock();
			}

		// L[Ƀf[^܂ŃTXyh
		if(!bKillSignal)pThread->SuspendThread();
		}

	::BON_TRACE(TEXT("CPtBoard::DispatchThread() End\n"));
}

void CPtBoard::DispatchStream(const DWORD *pdwData, const DWORD dwNum)
{
	#pragma pack(1)
	struct MICRO_PACKET
	{
		BYTE abyData[3];	// f[^	 2  1  0 ƂȂ
		BYTE Error : 1;		// G[tO
		BYTE Store : 1;		// TSpPbgJntO
		BYTE Count : 3;		// Xg[JE^
		BYTE Tuner : 3;		// Xg[CfbNX
	};
	#pragma pack()

	const MICRO_PACKET *pPacket = (const MICRO_PACKET *)pdwData;
	CPtTuner ** const ppTuner = &m_TunerList.front();	// vector̎QƂx

	// obt@TCYNA
	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		ppTuner[dwIndex]->m_StreamBuffer.ClearSize();
		}

	// }CNpPbgWJAfBXpb`
	for(register DWORD dwPos = 0UL ; dwPos < dwNum ; dwPos++, pPacket++){
		if((pPacket->Tuner >= 1) && (pPacket->Tuner <= 4)){
			const DWORD dwIndex = pPacket->Tuner - 1;
			for(DWORD dwData = 0UL ; dwData < 3UL ; dwData++){
				if(!ppTuner[dwIndex]->m_dwStreamCounter){
					// TSpPbg҂
					if(pPacket->Store && (pPacket->abyData[2 - dwData] == 0x47)){
						// TSpPbg擪
						ppTuner[dwIndex]->m_StreamBuffer.AddByte(pPacket->abyData[2 - dwData]);						
						ppTuner[dwIndex]->m_dwStreamCounter = 1UL;
						}
					}
				else if(ppTuner[dwIndex]->m_dwStreamCounter < 188UL){
					// TSpPbgXgA
					ppTuner[dwIndex]->m_StreamBuffer.AddByte(pPacket->abyData[2 - dwData]);
	
					if((++ppTuner[dwIndex]->m_dwStreamCounter) == 188UL){
						// TSpPbgXgA
						ppTuner[dwIndex]->m_dwStreamCounter = 0UL;
						}
					}
				}
			}		
		}

	// e`[iɏo
	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		if(ppTuner[dwIndex]->m_bIsStream){
			ppTuner[dwIndex]->OnTsStream(ppTuner[dwIndex]->m_StreamBuffer.GetData(), ppTuner[dwIndex]->m_StreamBuffer.GetSize());
			}
		}
}


/////////////////////////////////////////////////////////////////////////////
// PT`[iNX
/////////////////////////////////////////////////////////////////////////////

CPtTuner::CPtTuner(CPtBoard &Board, const DWORD dwTunerID, const Device::ISDB dwTunerType)
	: m_Board(Board)
	, m_dwTunerID(dwTunerID)
	, m_dwTunerType(dwTunerType)
	, m_bIsOpen(false)
	, m_bIsStream(false)
	, m_bIsLnbOn(false)
	, m_StreamBuffer(4096 * 511)
	, m_dwStreamCounter(0UL)
	, m_pTsReceiver(NULL)
{
	// Ȃ
}

CPtTuner::~CPtTuner()
{
	// `[iN[Y
	CloseTuner();
}

void CPtTuner::CloseTuner(void)
{
	// LNBdOFF
	SetLnbPower(false);

	// `[iN[Y
	m_Board.CloseTuner(this);
}

const bool CPtTuner::StartStream(ITsReceiver *pTsReceiver)
{
	// TSV[oCX^Xo^
	{
		CBlockLock AutoLock(&m_Lock);
		m_pTsReceiver = pTsReceiver;
	}

	// Xg[MX^[g
	return m_Board.StartStream(this);
}

void CPtTuner::StopStream(void)
{
	// TSV[oCX^Xo^
	{
		CBlockLock AutoLock(&m_Lock);
		m_pTsReceiver = NULL;
	}

	// Xg[MI
	m_Board.StopStream(this);
}

const bool CPtTuner::SetChannel(const DWORD dwChannel, const DWORD dwOffset)
{
	try{
		// Mgݒ
		if(m_Board.m_pDevice->SetFrequency(m_dwTunerID, m_dwTunerType, dwChannel, dwOffset) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTMgݒ莸s"));
		}
	catch(CBonException &Exception){
		// G[
		Exception.Notify();
		return false;
		}
	
	return true;
}

const bool CPtTuner::SetTsID(const WORD wTsID)
{
	DWORD dwCurTsID;

	try{
		// `[i̎ނ`FbN
		if(m_dwTunerType != Device::ISDB_S)throw ::BON_EXPECTION(TEXT("PTMTS IDݒΉ"));
	
		// MTS IDݒ
		if(m_Board.m_pDevice->SetIdS(m_dwTunerID, wTsID) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTMTS IDݒ莸s"));
		
		// fEFCg
		do{
			// TS ID擾
			::Sleep(1);
			if(m_Board.m_pDevice->GetIdS(m_dwTunerID, &dwCurTsID) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTMTS ID擾s"));
			
			// GfBAtɂȂ̂ŏC
			dwCurTsID = (DWORD)( ( (WORD)dwCurTsID << 8 ) | ( (WORD)dwCurTsID >> 8 ) );
			}
		while((WORD)dwCurTsID != wTsID);
		}
	catch(CBonException &Exception){
		// G[
		Exception.Notify();
		return false;
		}
	
	return true;
}

const bool CPtTuner::SetLnbPower(const bool bEnable)
{
	// LNBd
	return m_Board.SetLnbPower(this, bEnable);
}

const float CPtTuner::GetSignalLevel(void)
{
	DWORD dwCnr100 = 0UL, dwCurAgc = 0UL, dwMaxAgc = 0UL;
	
	try{
		// CNR擾
		if(m_Board.m_pDevice->GetCnAgc(m_dwTunerID, m_dwTunerType, &dwCnr100, &dwCurAgc, &dwMaxAgc) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT`[iCNR擾s"));
		}
	catch(CBonException &Exception){
		// G[
		Exception.Notify();
		return 0.0f;
		}
		
	// dBɕϊĕԂ
	return (float)dwCnr100 / 100.0f;
}

void CPtTuner::OnTsStream(const BYTE *pData, const DWORD dwSize)
{
	CBlockLock AutoLock(&m_Lock);

	// TSV[oɓ]
	if(m_pTsReceiver){
		m_pTsReceiver->OnTsStream(this, pData, dwSize);
		}
}
